/*
 *				**** DEMO ****
 *
 * This program demonstrates some features of the C language
 *	and how they are used.
 *
 */

/* include some definitions in header files */

#include "demo.h"

int
main(int argc,char **argv)
{
	int		i;		/* general purpose loop variable */
	int		opt;	/* option from user */
	char	buf[BUFSIZ];	/* input buffer */
	char	name[BUFSIZ];	/* user's name */

	debugsw = 0;	/* initialize debugging switch */
	persons = 0;	/* no persons loaded yet (see do_struct) */

	help_init("demo.hlp");

	/* first, display the command line arguments */

	printf("What is your name? ");		/* ask for name */
	gets(name);							/* get it from user */

	printf("Hello, %s.  Welcome to the C language demo program.\n", name);

	printf("%d command line arguments were passed:\n", argc-1);
	for (i=0; i < argc; i++)
	{		/* loop thru list of command line arguments */
		printf("Argument %d: >>%s<<\n", i, argv[i]);
	}

	/* now, ask what the user would like to do */

	do
	{
		printf("Q\tQuit demo program\n\n");
		printf("A\tExperiment with function arguments.\n");
		printf("B\tExperiment with Boolean operations.\n");
		printf("D\tSet value of debugging switch\n");
		printf("M\tExperiment with Math operations.\n");
		printf("P\tExperiment with pointers.\n");
		printf("S\tExperiment with structures.\n");
		printf("F\tExperiment finding entries in lists.\n");
		printf("\n?\tDisplay a help message.\n");
		printf("C\tDisplay copyright and porting information.\n");
		printf("\nEnter option: ");

		gets(buf);						/* read answer from user */
		for (i=0; buf[i] == ' '; i++);	/* find non-blank */
		opt = buf[i];					/* get option */

		if (debugsw>0) printf("opt=%d, i=%d, buf=>>%s<<\n", opt, i, buf);


		/* now, act on the requested option */

		switch(opt)
		{		/* branch on option */
			case 'A':			/* arguments */
			case 'a':
				do_args(name);
				break;

			case 'b':
			case 'B':
				do_boolean(name);	/* do boolean experiments */
				break;				/* leave switch statement */

			case 'c':
			case 'C':
				help("(c)");
				break;

			case 'd':
			case 'D':
				printf("Enter new value for debug switch: ");
				gets(buf);
				debugsw = atoi(buf);
				printf("Debug switch now set to %d.\n", debugsw);
				break;

			case 'f':				/* find - search */
			case 'F':
				do_srch(name);
				break;

			case 'M':				/* math */
			case 'm':
				do_math(name);		/* do the math experiments */
				break;				/* exit switch statement */

			case 'p':
			case 'P':
				do_pointers(name);	/* do pointers experiments */
				break;

			case 'q':
			case 'Q':
				printf("Are you sure? (y or n):");	/* double check */
				gets(buf);			/* get response from user */
				for (i = 0; buf[i] == ' '; i++);	/* find non-blank */
				if (buf[i] == 'y' || buf[i] == 'Y') opt = 0;
				break;

			case 's':
			case 'S':
				do_struct(name);	/* do structure experiments */
				break;

			case '?':
				help("demo");		/* get help */
				break;

			default:
				printf("I'm sorry, %s, but option %c is invalid.\n",
						name, opt);
				opt = -1;
				break;
		}
	} while (opt != 0);

		/* all done, say good-night, Gracie */
	printf("Goodbye %s, thanks for using this program.\n", name);
	printf("\n\tAnd good luck with C!\n");

	help_close();
	return(0);
}

void
do_continue(void)
{		/* force user to press Enter to continue */
	char buf[BUFSIZ];

	printf("Press ENTER to continue: ");
	gets(buf);
}

