/*
 *				**** DO STRUCT ****
 *
 * This routine allows the user to experiment with structures
 *
 */

#include "demo.h"
#include <errno.h>
/*
 * NOTE:
 *
 * The definition for the structures used in these routines can be
 *	found in the header file demo.h
 *
 */

void do_struct(char *);
void add_struct(char *);
void clean_struct(char *);
void del_struct(char *);
void ed_struct(char *);
void read_struct(char *);
char *load_mem(FILE *);
void clean_up(char *);
void write_struct(char *);
void write_ent(int,FILE *);
void zap_struct(char *);
void list_struct(char *);
void list_ent(int);
char *fmt_date(struct date_str);
void print_mem(char *,char *);
void free_struct(struct person *);
void ent_struct(struct person *);
void parse_date(struct date_str *,char *);
void undel_struct(char *);
char *new_mem(char *,char *);
void move_struct(int,int);

void
do_struct(char *who)
{		/* experiment with structures */
	char	*p;		/* useful pointer */
	char	buf[BUFSIZ];	/* input buffer */
	int		opt;			/* option from user */

	if (debugsw>0) printf("Entering do_struct, argument >>%s<<\n", who);

	printf("Hello, %s.  Welcome to the structure routines.\n", who);

	do
	{
		printf("\nOptions are:\n\n");
		printf("\tQ\tReturn to main menu.\n\n");
		printf("\tA\tAdd new entries to structure.\n");
		printf("\tC\tClean out deleted entries from structure.\n");
		printf("\tD\tDelete entries from strucure.\n");
		printf("\tE\tEdit entry in structure.\n");
		printf("\tL\tList entries in structure.\n");
		printf("\tR\tRead entries from file.\n");
		printf("\tU\tUndelete entry in structure.\n");
		printf("\tW\tWrite entries to file.\n");
		printf("\tZ\tZap entire structure empty.\n");
		printf("\t?\tDisplay a help message.\n");
		printf("\nEnter option: ");

		gets(buf);
		for (p = buf; *p == ' '; p++);	/* find non-blank */
		switch (opt = *p)
		{		/* branch on value of opt */
			case 'a':
			case 'A':	/* do add */
				if (persons < MAXPEOPLE-1) add_struct(who);
				else
				{		/* the structure is full at present */
					printf("No more room in structure.\n");
					printf("Try the clean option.\n");
				}
				break;

			case 'c':
			case 'C':
				clean_struct(who);
				break;

			case 'd':
			case 'D':
				del_struct(who);
				break;

			case 'e':
			case 'E':
				ed_struct(who);
				break;

			case 'l':
			case 'L':
				list_struct(who);
				break;

			case 'q':
			case 'Q':
				opt = 0;		/* signal exit */
				break;

			case 'r':
			case 'R':
				read_struct(who);
				break;

			case 'u':
			case 'U':
				undel_struct(who);
				break;

			case 'w':
			case 'W':
				write_struct(who);
				break;

			case 'z':
			case 'Z':
				zap_struct(who);
				break;

			case '?':
				help("struct");
				break;

			default:
				printf("Sorry, %s, but option %s is invalid.\n", who, buf);
				printf("Please try again.\n");
		}
	} while (opt);

	if (debugsw>0) printf("Leaving do_struct.\n");
}

void
add_struct(char *who)
{		/* add user into structure */
	if (debugsw>0) printf("Entering add_struct, argument >>%s<<\n", who);

	if (persons >= MAXPEOPLE-1) return;	/* should never really get here */

	ent_struct(&people[persons]);	/* enter info into structure */
	people[persons].p_flag = P_USED;	/* mark the entry used */

	persons++;						/* location of next free entry */

	if (debugsw>0) printf("Leaving add_struct.\n");
}

void
clean_struct(char *who)
{		/* compress and clean structure */
	int	new, old;		/* new and old pointers */

	if (debugsw>0) printf("Entering clean_struct, argument >>%s<<\n", who);

	for (old=new=0; old < persons; old++)
	{		/* loop thru old entries and move them up to new entries */
		if (people[old].p_flag == P_USED)
		{			/* entry in use, save it */
			if (new != old) move_struct(new,old);	/* move up entry */
			new++;		/* count this entry */
		}
	}

	persons = new;		/* reset counter to next free entry */
	printf("Structure now has %d entries in use.\n", persons);

	if (debugsw>0) printf("Leaving clean_struct.\n");
}

void
del_struct(char *who)
{
	char	buf[BUFSIZ];		/* input buffer */
	int		ent;				/* entry to delete */

	if (debugsw>0) printf("Entering del_struct, argument >>%s<<\n", who);

	printf("%d entries in structure.\n", persons);
	printf("Enter entry to delete: ");
	ent = atoi(gets(buf));		/* get entry number */
	if (ent >= 0 && ent < persons)
	{		/* valid entry (so far) */
		if (people[ent].p_flag == P_USED)
		{		/* entry in use, double check */
			printf("Entry to be deleted:\n");
			list_ent(ent);		/* list entry */
			printf("\nDelete this entry? (y or n): ");
			if (*gets(buf) == 'y' || *buf == 'Y')
			{
				people[ent].p_flag = P_DEL;
				printf("Entry %d deleted.\n", ent);
			} else printf("Entry %d not deleted.\n", ent);
		} else printf("Entry %d is not an active entry.\n", ent);
	} else printf("Sorry %s, but %d is not a valid entry number.\n", ent);

	if (debugsw>0) printf("Leaving del_struct.\n");
}

void
ed_struct(char *who)
{
	char	buf[BUFSIZ];		/* user input buffer */
	int		ent;				/* entry to edit */

	if (debugsw>0) printf("Entering ed_struct, argument >>%s<<\n", who);

	printf("There are %d entries in structure.\n", persons);
	printf("Enter number of entry to edit: ");
	ent = atoi(gets(buf));
	if (ent >= 0 && ent < persons) ent_struct(people+ent);
	else
	{		/* invalid entry */
		printf("Sorry, %s, but %d is not a valid entry.\n", who, ent);
	}

	if (debugsw>0) printf("Leaving ed_struct.\n");
}

void
read_struct(char *who)
{		/* read entries from file */
	char	buf[BUFSIZ];		/* input buffer */
	FILE	*fp;				/* input file pointer */
	char *load_mem();			/* char ptr function used here */

	if (debugsw>0) printf("Entering read_struct, argument >>%s<<\n", who);

	printf("Enter input file name: ");
	gets(buf);
	if ((fp = fopen(buf,"r")) == NULL)
	{		/* can't open file */
		printf("Can't open input file %s.\n", buf);
		perror("read_struct");
	} else
	{		/* do append */
		while (fgets(buf,sizeof(buf),fp) != NULL)
		{
			clean_up(buf);
			if (strlen(buf) < IDSIZE) strcpy(people[persons].p_id,buf);
			else *people[persons].p_id = '\0';
			printf("Reading entry %d (%s)\n", persons, people[persons].p_id);
			people[persons].p_flag = P_USED;	/* signal entry used */

			people[persons].p_lname = load_mem(fp);
			people[persons].p_fname = load_mem(fp);
			people[persons].p_addr  = load_mem(fp);
			people[persons].p_city  = load_mem(fp);
			people[persons].p_state = load_mem(fp);
			people[persons].p_zip   = load_mem(fp);

			fgets(buf,sizeof(buf),fp);		/* date string */
			parse_date(&people[persons].p_bday,buf);
			persons++;						/* one more entry */
		}		/* end reading person entry */
	}

	if (debugsw>0) printf("Leaving read_struct.\n");
}

char *
load_mem(FILE *fp)
{		/* read file & allocate memory for it */
	char	buf[BUFSIZ];		/* input buffer */
	char	*p;					/* pointer for memory */

	if (debugsw>0) printf("Entering load_mem, argument %x.\n", fp);

	fgets(buf,sizeof(buf),fp);	/* read line */
	if (*buf != '\0')
	{		/* something read - get memory */
		clean_up(buf);			/* get rid of CR/LF */
		p = (char *)malloc(strlen(buf)+1);
		strcpy(p,buf);
	} else p = NULL;

	if (debugsw>0) printf("Leaving load_mem.\n");

	return p;
}

void
clean_up(char *str)
{
	char	*p;		/* pointer for scanning */

	for (p = str; *p != '\0' &&*p != '\r' && *p != '\n'; p++);
	*p = '\0';
}

void
write_struct(char *who)
{		/* write entries to file */
	char	buf[BUFSIZ];		/* input buffer */
	FILE	*fp;				/* output file pointer */
	int		i;					/* loop variable */

	if (debugsw>0) printf("Entering write_struct, argument >>%s<<\n", who);

	printf("Enter name of output file: ");
	gets(buf);
	if ((fp = fopen(buf,"w")) == NULL)
	{		/* can't open */
		printf("Can't open file %s for writing.  Command cancelled.\n", buf);
		perror("write_struct");
	} else
	{		/* file opened, write everything */
		for (i=0; i < persons; i++) write_ent(i,fp);	/* write entries */
		fclose(fp);
	}

	if (debugsw>0) printf("Leaving write_struct.\n");
}

void
write_ent(int ent,FILE *fp)
{		/* write entry from people to file */
	char *fmt_date();

	if (debugsw>0) printf("Entering write_ent, arguments %d & %x.\n",
							ent, fp);

	if (people[ent].p_flag == P_USED)
	{		/* valid entry - save */
		printf("Writing entry %d (ID %s)\n", ent, people[ent].p_id);
		fputs(people[ent].p_id,fp);
		fputc('\n',fp);
		if (people[ent].p_lname != NULL) fputs(people[ent].p_lname,fp);
		fputc('\n',fp);
		if (people[ent].p_fname != NULL) fputs(people[ent].p_fname,fp);
		fputc('\n',fp);
		if (people[ent].p_addr  != NULL) fputs(people[ent].p_addr,fp);
		fputc('\n',fp);
		if (people[ent].p_city  != NULL) fputs(people[ent].p_city,fp);
		fputc('\n',fp);
		if (people[ent].p_state != NULL) fputs(people[ent].p_state,fp);
		fputc('\n',fp);
		if (people[ent].p_zip   != NULL) fputs(people[ent].p_zip,fp);
		fputc('\n',fp);
		fputs(fmt_date(people[ent].p_bday),fp);
		fputc('\n',fp);
	} else printf("Skipping entry %d (ID %s)\n", ent, people[ent].p_id);

	if (debugsw>0) printf("Leaving write_ent.\n");
}

void
zap_struct(char *who)
{
	char	buf[BUFSIZ];		/* input buffer */
	int		i;					/* loop variable */

	if (debugsw>0) printf("Entering zap_struct, argument >>%s<<\n", who);

	printf("**** You have asked to clear the entire structure ***\n");
	printf("**** %s, are you sure you want to do that? (y or n): ", who);
	gets(buf);
	if (*buf == 'y' || *buf == 'Y')
	{		/* yep, it looks like we are going to zap em all */
		printf("Clearing entire structure ...\n");
		for (i=0; i < persons; i++)
		{		/* clear each entry */
			free_struct(people+i);		/* free up memory */
			people[i].p_flag = P_EMPTY;	/* signal empty */
		}
		persons = 0;					/* no entries */
		printf("Structure cleared.\n");
	}

	if (debugsw>0) printf("Leaving zap_struct.\n");
}

void
list_struct(char *who)
{		/* list entry or entries in structure */
	char	buf[BUFSIZ];		/* input buffer */
	int		elow, ehigh;		/* low and high entries */
	int		i;					/* loop variable */

	if (debugsw>0) printf("Entering list_struct, argument >>%s<<\n", who);

	printf("%d entries in structure.\n", persons);
	printf("Enter low entry to list: ");
	elow = atoi(gets(buf));		/* get low limit */
	printf("Enter high entry to list: ");
	ehigh = atoi(gets(buf));	/* get high limit */

	if (elow >= 0 && elow < persons && ehigh >= elow && ehigh < persons)
	{		/* valid range, do it */
			/* note that we pass the index into the structure entry */
		for (i = elow; i <= ehigh; i++)
		{		/* loop thru & list entries */
			list_ent(i);
			printf("\nPress Enter to continue: ");
			gets(buf);
		}
	} else
	{		/* invalid range */
		printf("Sorry, %s, but %d to %d is not a valid range.\n",
				who, elow, ehigh);
	}

	if (debugsw>0) printf("Leaving list_struct.\n");
}

void
list_ent(int ent)
{		/* list the values of an entry */
	char *fmt_date();	/* function used in here */

	if (debugsw>0) printf("Entering list_ent, argument %d.\n", ent);

	printf("Entry %d, User %s", ent, people[ent].p_id);
	if (people[ent].p_flag == P_USED) printf(" (Used):\n");
	else if (people[ent].p_flag == P_DEL) printf(" (Deleted):\n");

	print_mem("LName:",people[ent].p_lname);
	print_mem("FName:",people[ent].p_fname);
	print_mem("Addr:",people[ent].p_addr);
	print_mem("City:",people[ent].p_city);
	print_mem("State:",people[ent].p_state);
	print_mem("Zip:",people[ent].p_zip);
	print_mem("BDay:",fmt_date(people[ent].p_bday));
}

char *
fmt_date(struct date_str dstr)
{
	static	char	outstr[12];		/* output string */
	char	buf[5];					/* used to format date */

	if (dstr.date_day == 0) outstr[0] = '\0';
	else
	{		/* valid(?) date, build date string */
		sprintf(buf,"%2.2d",dstr.date_mon);	/* convert month to str */
		strcpy(outstr,buf);		/* copy month string */
		strcat(outstr,"/");			/* add slash */
		sprintf(buf,"%2.2d",dstr.date_day);	/* convert day to str */
		strcat(outstr,buf);			/* add day string */
		strcat(outstr,"/");			/* add slash */
		sprintf(buf,"%d",dstr.date_yr);	/* convert year to str */
		strcat(outstr,buf);			/* add year string */
	}

	if (debugsw>0) printf("Leaving fmt_date, outstr >>%s<<\n", outstr);

	return outstr;
}

void
print_mem(char *lbl,char *addr)
{		/* print label & memory, handling NULL address */
	printf("\t%-8s",lbl);	/* print label*/
	if (addr != NULL) printf("%s\n", addr);	/* print content */
	else printf("[no value]\n");
}

void
free_struct(struct person *pstr)
{		/* free memory up associated with this entry */
	if (debugsw>0) printf("Entering free_struct,argument %x.\n", pstr);

	*pstr->p_id = '\0';		/* clear ID field */

		/* now clear allocated memory fields */
	if (pstr->p_lname != NULL) free(pstr->p_lname);
	if (pstr->p_fname != NULL) free(pstr->p_fname);
	if (pstr->p_addr != NULL) free(pstr->p_addr);
	if (pstr->p_city != NULL) free(pstr->p_city);
	if (pstr->p_state != NULL) free(pstr->p_state);
	if (pstr->p_zip != NULL) free(pstr->p_zip);
	pstr->p_bday.date_yr = pstr->p_bday.date_mon = pstr->p_bday.date_day = 0;

	/* noiw zap pointers */
	pstr->p_lname = pstr->p_fname = pstr->p_addr = pstr->p_city =
		pstr->p_state = pstr->p_zip = NULL;

	if (debugsw>0) printf("Leaving free_struct.\n");
}

void
ent_struct(struct person *dstr)
{		/* enter values for entry */
	char	buf[BUFSIZ];	/* input buffer */
	char	*p;				/* used for allocating memory */
	char	*new_mem();		/* function returning character string */

	if (debugsw>0) printf("Entering ent_struct, addr %x.\n", dstr);

	if (dstr->p_flag != P_EMPTY)
	{
		printf("Entry user ID is currently %s, edit? (y or n): ", dstr->p_id);
		gets(buf);
	} else
	{		/* new entry */
		printf("Entering new entry? (y or n): ");
		gets(buf);
	}

	if (*buf == 'y' || *buf == 'Y')
	{		/* do the edit */
		printf("The old values will be displayed for each field.\n");
		printf("Enter the new value for each field.\n");
		printf("To leave a field unchanged, just press Enter.\n");
		if ((p = new_mem("ID:",dstr->p_id)) != NULL)
		{		/* new ID field requested */
			if (strlen(p) < IDSIZE) strcpy(dstr->p_id,p);
			else printf("ID field unchanged (new field too long).\n");
			free(p);		/* don't need memory any more */
		}

		if ((p = new_mem("LName:",dstr->p_lname)) != NULL)
		{		/* new value entred - get rid of old and replace */
			if (dstr->p_lname != NULL) free(dstr->p_lname);	/* free old */
			dstr->p_lname = p;			/* new value */
		}

		if ((p = new_mem("FName:",dstr->p_fname)) != NULL)
		{
			if (dstr->p_fname != NULL) free(dstr->p_fname);
			dstr->p_fname = p;
		}

		if ((p = new_mem("Addr:",dstr->p_addr)) != NULL)
		{
			if (dstr->p_addr != NULL) free(dstr->p_addr);
			dstr->p_addr = p;
		}

		if ((p = new_mem("City:",dstr->p_city)) != NULL)
		{
			if (dstr->p_city != NULL) free(dstr->p_city);
			dstr->p_city = p;
		}

		if ((p = new_mem("State:",dstr->p_state)) != NULL)
		{
			if (dstr->p_state != NULL) free(dstr->p_state);
			dstr->p_state = p;
		}

		if ((p = new_mem("Zip:",dstr->p_zip)) != NULL)
		{
			if (dstr->p_zip != NULL) free(dstr->p_zip);
			dstr->p_zip = p;
		}

		sprintf(buf,"%2.2d/%2.2d/%2.2d",dstr->p_bday.date_mon,
				dstr->p_bday.date_day, dstr->p_bday.date_yr);
		if ((p = new_mem("BDay:",buf)) != NULL)
		{		/* new birthday */
			parse_date(&dstr->p_bday,p);	/* parse date & update field */
			free(p);		/* don't need memory any more */
		}

	}		/* end updating entry */

	if (debugsw>0) printf("Leaving ent_struct.\n");

}

void
parse_date(struct date_str *out,char *in)
{		/* parse date and place in output structure */
	char	*p, *q;			/* used for parsing */

	if (debugsw>0) printf("Entering parse_date, args %x & %s\n", out, in);

	for (p = q = in; *p != '\0' && *p != '/'; p++);	/* end of month */
	if (*p != '\0') *p++ = '\0';		/* terminate field */
	out->date_mon = atoi(q);			/* convert month to binary */
	for (q=p; *p != '\0' && *p != '/'; p++);		/* end of day */
	if (*p != '\0') *p++ = '\0';		/* terminate field */
	out->date_day = atoi(q);			/* convert day to binary */
	out->date_yr  = atoi(p);			/* convert year to binary */

	if (debugsw>0) printf("Leaving parse_date, out=%d/%d/%d\n",
							out->date_mon, out->date_day, out->date_yr);
}

void
undel_struct(char *who)
{
	char	buf[BUFSIZ];		/* input buffer */
	int		ent;				/* entry to undelete */

	if (debugsw>0) printf("Entering undel_struct, argument >>%s<<\n", who);

	printf("There are %d entries in structure.\n");
	printf("Enter entry to undelete: ");
	ent = atoi(gets(buf));		/* get entry number */
	if (ent >= 0 && ent < persons)
	{		/* valid number */
		if (people[ent].p_flag ==P_DEL)
		{		/* yes, the entry was deleted */
			people[ent].p_flag = P_USED;
		} else printf("Sorry, %s, entry %d is not deleted.\n", who, ent);
	} else printf("Sorry, %s, but %d is not a valid entry number.\n",
					who, ent);

	if (debugsw>0) printf("Leaving undel_struct.\n");
}

char *
new_mem(char *lbl,char *addr)
{		/* get new memory */
	char	buf[BUFSIZ];		/* input buffer */
	char	*ret;				/* return pointer */

	printf("\t%s\t", lbl);		/* print label */
	if (addr == NULL || *addr == '\0') printf("[no value]: ");
	else printf("(%s): ", addr);	/* old value */
	gets(buf);					/* get new value */
	if (*buf != '\0')
	{		/* new value entered */
		ret = (char *) malloc(strlen(buf)+1);	/* get memory for response */
		strcpy(ret,buf);				/* save new value */
	} else ret = NULL;					/* no new value */

	if (debugsw>0)
	{
		printf("Leaving new_mem, ");
		if (ret != NULL) printf("returning >>%s<<\n", ret);
		else printf("no value returned.\n");
	}

	return ret;
}

void
move_struct(int targ,int src)
{		/* move a structure from 'src' to 'targ'*/
	if (debugsw>0) printf("Entering move_struct, src=%d, targ=%d.\n",
						src, targ);

		/* copy fields that need no additional memory */
	people[targ].p_flag = people[src].p_flag;		/* usage flag */
	strcpy(people[targ].p_id,people[src].p_id);		/* user ID */

		/* copy fields with pointers */
	people[targ].p_lname = people[src].p_lname;
	people[targ].p_fname = people[src].p_fname;
	people[targ].p_addr = people[src].p_addr;
	people[targ].p_city = people[src].p_city;
	people[targ].p_state = people[src].p_state;
	people[targ].p_zip = people[src].p_zip;

		/* clear out old structure */
	people[src].p_flag = P_EMPTY;
	*people[src].p_id = '\0';
	people[src].p_lname = people[src].p_fname = people[src].p_addr =
		people[src].p_city = people[src].p_state = people[src].p_zip = NULL;

	people[src].p_bday.date_mon = people[src].p_bday.date_day =
		people[src].p_bday.date_yr = 0;

	if (debugsw>0) printf("Leaving move_struct.\n");
}
