/*
 *				**** DO BOOLEAN ****
 *
 * This routine allows the user to perform experiments with booleans
 *
 */

#include "demo.h"

void do_boolean(char *);
int get_char(char *);
void put_vals(int,int);
void char_hdr(void);
void put_char(char *,int);
void put_bool(int,int);
char * bits(int);
void do_continue(void);

void
do_boolean(char *who)
{
	char	buf[BUFSIZ];			/* input buffer */
	int		v1, v2;				/* boolean variables */
	int		opt;					/* option from user */

	if (debugsw>5) printf("Entering do_boolean, argument >>%s<<\n", who);

	printf("Hello, %s.  Welcome to the boolean routines.\n",who);

	do
	{
		printf("\nBoolean Options are:\n");
		printf("\t1\tSet value of first variable.\n");
		printf("\t2\tSet value of second variable.\n");
		printf("\t3\tDisplay values of variables (by bits).\n");
		printf("\t4\tPerform boolean operations on variables.\n");
		printf("\n\t9\tDisplay a help message.\n");
		printf("\t99\tReturn to Main Menu.\n");
		printf("Enter option: ");

		gets(buf);
      if (buf[0] == 'q' || buf[0] == 'Q') break; /* exit */
		switch(opt = atoi(buf))
		{		/* branch depends on value of option */
			case 0:		/* exit routine */
				break;		/* just exit */
			case 1:		/* set first variable */
				v1 = get_char("Enter value for first variable: ");
				break;
			case 2:		/* set second variable */
				v2 = get_char("Enter value for second variable: ");
				break;
			case 3:		/* display values */
				put_vals(v1,v2);
				do_continue();
				break;
			case 4:		/* perform boolean */
				put_vals(v1,v2);
				put_bool(v1,v2);
				do_continue();
				break;
			case 9:		/* display help message */
				help("bool");
				break;
         case 99: /* exit routine */
         	printf("Are you sure you want to exit? (y or n): ");
            gets(buf);
            if (*buf != 'y') opt = 0;	/* don't exit */
         	break;
			default:	/* invalid option */
				printf("Sorry, %s, but %d is not a valid option.\n",
						who, opt);
				printf("Try again.\n");
				break;
		}
	} while (opt != 99);

	if (debugsw>5) printf("Leaving do_boolean.\n");
}

int
get_char(char *prompt)
{		/* extract character value */
	char	buf[BUFSIZ];	/* input buffer from user */
	int		ret;			/* return value */

	if (debugsw>0) printf("Entering get_char, argument >>%s<<\n", prompt);

	printf("Enter single character value (e.g. A, q, etc.)\n");
	printf("To enter hex value, enter more than one character (e.g., 0f).\n");
	printf("%s", prompt);
	gets(buf);
	if (*(buf+1) == '\0') ret = *buf;
	else if ((ret = atohex(buf)) < 0)
	{		/* invalid hex character */
		printf("%s is an invalid hex string.  Try again.\n");
		ret = get_char(prompt);		/* recursive call to get another value */
	}

	if (debugsw>0) printf("Leaving get_char, value %d.\n", ret);
	return ret;
}

void
put_vals(int v1, int v2)
{		/* print header and values */

	if (debugsw>0) printf("Entering put_vals, arguments %d & %d.\n", v1, v2);

	char_hdr();
	put_char("V1",v1);
	put_char("V2",v2);

	if (debugsw>0) printf("Leaving put_vals\n");
}

void
char_hdr(void)
{		/* print character header string */

	printf("Var.\tHex.\tDec.\tBin.\t\tChar.\n");
}

void
put_char(char *lbl, int val)
{		/* dislay value in several formats */
	char *bits();	/* string function used here */

	printf("%s\t%2.2X\t%d\t%8.8s\t%c\n", lbl, val, val, bits(val), val);
}

void
put_bool(int v1, int v2)
{

	if (debugsw>0) printf("Entering put_bool, arguments %d & %d.\n", v1, v2);

	put_char("AND",v1&v2);
	put_char("OR",v1|v2);
	put_char("XOR",v1^v2);
	put_char("!v1",!v1);
	put_char("!v2",!v2);
	put_char("~v1",~v1);
	put_char("~v2",~v2);

	if (debugsw>0) printf("Leaving put_bool.\n");
}

char *
bits(int val)
{		/* express input value as bit string */
	static	char	outv[9];	/* output string */
	int		i;			/* loop variable */

	if (debugsw>0) printf("Entering bits, argument %d.\n", val);

	for (i=0; i < 8; i++)
	{		/* look at each bit */
		if (debugsw>5) printf("i=%d, val=%x.\n", i, val);
		if (val & 1) outv[7-i] = '1';
		else outv[7-i] = '0';
		if (debugsw>5) printf("outv[%d]=%c.\n", 7-i, outv[7-i]);
		val >>= 1;
	}

	outv[8] = '\0';		/* terminate string */

	if (debugsw>0) printf("Leaving bits, value >>%s<<\n", outv);

	return outv;
}

