/*
 *				**** DO POINTER ****
 *
 * This routine allows the user to experiment with pointers
 *
 */

#include <ctype.h>

#include "demo.h"

void do_int(char *who);
void do_look(char *who);
void do_mem(char *who);
void do_str(char *who);
void dump_int(void);
void dump_str(void);

/* static variables outside of functions are global for this file only */

static	char	str[BUFSIZ];	/* string to work with */
static	char	*strp;			/* character pointer */
static	int	int1;				/* integer to work with */
static	int	int2;				/* another integer to work with */
static	int	*intp;			/* integer pointer */


void do_pointers(person)
char	*person;		/* name of user */
{		/* experiment with pointers */
	char	*p;			/* integer pointer */
	char	buf[BUFSIZ];	/* input buffer */
	int		opt;		/* option from user */

	if (debugsw>5) printf("Entering do_pointers, argument >>%s<<\n", person);

	printf("Hello %s! Welcome to the pointer routine.\n", person);

   /* initialize values */
   strcpy(str, "Test string");
   strp = str;
   int1 = 1;
   int2 = 2;
   intp = &int1;

	printf("Working string is at address %x, integer variable at %x.\n",
			&str, &int1);

	do
	{		/* display menu, get an option, then act on the option */
		printf("Options are:\n");
		printf("\tI\tExperment with integer pointers.\n");
		printf("\tL\tLook at memory.\n");
		printf("\tM\tSet memory locations.\n");
		printf("\tS\tExperiment with string pointers.\n");
		printf("\n\t?\tDisplay help message.\n");
		printf("\n\tQ\tQuit pointer routine.\n");
		printf("\nEnter option: ");

		gets(buf);	/* get option from user */
		for (p = buf; *p == ' '; p++);	/* find non-blank */
		opt = *p;						/* get option */

		switch(opt)
		{		/* branch depending on value of opt */
			case 'i':
			case 'I':
				do_int(person);
				break;
			case 'l':
			case 'L':
				do_look(person);
				break;
			case 'm':
			case 'M':
				do_mem(person);
				break;
			case 's':
			case 'S':
				do_str(person);
				break;
			case 'q':
			case 'Q':
				opt = 0;
				break;
			case '?':
				help("ptr");
				break;
			default:
				printf("Sorry, %s, but that is not a valid option.\n",
						person);
				printf("Please try again.\n");
				break;
		}
	} while (opt);
}

void
do_int(char *who)
{		/* experiment with integer pointers */
	char	buf[BUFSIZ];		/* work buffer */
	int		opt;				/* option number */

	if (debugsw>0) printf("Entering do_int, argument >>%s<<\n", who);

	do
	{
		dump_int();		/* report on integer */
		printf("\nOptions are:\n");
		printf("\t0\tExit Pointer routine.\n");
		printf("\t1\tSet Address of integer pointer.\n");
		printf("\t2\tSet Integer Value via pointer.\n");
		printf("\t3\tSet Integer Value of int1 Directly.\n");
		printf("\t4\tSet Integer Value of int2 Directly.\n");
		printf("\n\t9\tDisplay help message.\n");
		printf("\nEnter option: ");

		gets(buf);
		switch(opt = atoi(buf))
		{		/* check option */
			case 0:		/* exit */
				break;
			case 1:		/* set address */
				printf("Enter hex address for integer pointer: ");
				gets(buf);
				intp = (int *)atohex(buf);		/* convert hex string to int */
				break;
			case 2:		/* set integer via pointer */
				printf("Enter decimal value for integer via pointer: ");
				gets(buf);
				*intp = atoi(buf);		/* set via pointer */
				break;
			case 3:		/* set integer directly */
				printf("Enter decimal value for int1: ");
				gets(buf);
				int1 = atoi(buf);
				break;
			case 4:		/* set integer int2 directly */
				printf("Enter decimal value for int2: ");
				int2 = atoi(gets(buf));
				break;
			case 9:		/* display help message */
				help("iptr");
				break;
			default:	/* unknown option */
				printf("I'm sorry %s, but option %d is not valid.\n",
						who, opt);
		}
	} while (opt);

	if (debugsw>0) printf("Leaving do_int.\n");
}

void
do_look(char *who)
{
	char	*p;		/* pointer variable for looping */
	char	*beg;	/* starting location */
	char	*end;	/* ending location */
	char	buf[BUFSIZ];	/* input buffer */
   int   cnt=0;		/* line counter */

	if (debugsw) printf("Entering do_look, argument >>%s<<\n", who);

	dump_int();		/* display values of integer pointers */
	dump_str();		/* display values of string pointers */

	printf("Enter beginning address to look: ");
	beg = (char *)atohex(gets(buf));

	printf("Enter ending hex address to look: ");
	end = (char *)atohex(gets(buf));

	printf("Addr.\tHex\tDec\tChar\n");
	for (p = beg; p <= end; p++)
	{
		printf("%6.6x\t%2.2x\t%3d\t%c\n", p, *p, *p, *p);
      if (++cnt > 20)
      {	/* pause every 20 lines */
      	printf("Press Enter to continue: ");
         gets(buf);
         if (*buf == 'q' || *buf == 'Q') break;
         cnt = 0;
      }
	}

	printf("Press Enter to continue: ");
	gets(buf);

	if (debugsw) printf("Leaving do_look.\n");
}

void
do_mem(who)
char	*who;
{
	char	*beg;				/* beginning location of set */
	char	*end;				/* ending location of set */
	char	*mem;				/* current location of memory */
	char	buf[BUFSIZ];		/* response from user */
	int		val;				/* new value */

	if (debugsw) printf("Entering do_mem, argument >>%s<<\n",who);

	dump_int();					/* report on integers */
	dump_str();					/* report on strings */
	printf("Enter beginning address of set: ");
	beg = (char *) atohex(gets(buf));

	printf("Enter ending address of set: ");
	end = (char *) atohex(gets(buf));

	printf("Enter a single character value for each location (e.g., A, q, etc.\n");
	printf("To specify a hex value, enter at least two characters (e.g. 0f)\n");
	printf("Press Enter to leave location unchanged.\n");
   printf("To quit, enter the word: quit\n");
	printf("Addr.\tHex\tDec\tChar\n");

	for (mem = beg; mem <= end; mem++)
	{		/* loop thru memory */

		printf("%6.6x\t%x\t%d\t%c\t:", mem, *mem, *mem, *mem);
		gets(buf);			/* get possible new value */
      if (strcmp(buf, "quit") == 0) break;	/* quit */
		if (*buf != '\0')
		{   	/* new value entered, set memory */
			if (*(buf+1) == '\0') *mem = *buf;	/* single character */
			else if ((val = atohex(buf)) >= 0) *mem = val;
			else
			{		/* invalid hex value - try again */
				printf("%s is not a valid hex string.  Try again.\n", buf);
				mem--;		/* back up mem so it will point to same place */
				continue;
			}
		}

	}

	printf("Press Enter to continue: ");
	gets(buf);

	if (debugsw) printf("Leaving do_mem.\n");
}

void
do_str(who)
char	*who;		/* user's name */
{
	char	buf[BUFSIZ];		/* work buffer */
	int		opt;				/* option number */

	if (debugsw>0) printf("Entering do_str, argument >>%s<<\n", who);

	do
	{
		dump_str();		/* report on integer */
		printf("\nOptions are:\n");
		printf("\t0\tExit String Pointer routine.\n");
		printf("\t1\tSet Address of string pointer.\n");
		printf("\t2\tSet String via pointer.\n");
		printf("\t3\tSet String Value.\n");
		printf("\n\t9\tDisplay a help message.\n");
		printf("\nEnter option: ");

		gets(buf);
		switch(opt = atoi(buf))
		{		/* check option */
			case 0:		/* exit */
				break;
			case 1:		/* set address */
				printf("Enter hex address for string pointer: ");
				gets(buf);
				strp = (char *)atohex(buf);		/* convert hex string to int */
				break;
			case 2:		/* set string via pointer */
				printf("Enter value for string via pointer: ");
				gets(buf);
				strcpy(strp,buf);				/* set via pointer */
				break;
			case 3:		/* set string directly */
				printf("Enter value for string: ");
				gets(buf);
				strcpy(str,buf);
				break;
			case 9:		/* display a help message */
				help("sptr");
				break;
			default:	/* unknown option */
				printf("I'm sorry %s, but option %d is not valid.\n",
						who, opt);
		}
	} while (opt);

	if (debugsw>0) printf("Leaving do_str.\n");
}

int
atohex(char *str)
{
	int		val;	/* resulting value */

	for (val = 0; *str != '\0'; str++)
	{
		val *= 16;		/* shift up for new digit */
		if (isdigit(*str)) val += *str - '0';	/* add decimal value */
		else if(isxdigit(*str))
				if (islower(*str)) val += *str - 'a' + 10;
				else val += *str - 'A' + 10;
		else if (!isspace(*str))
		{		/* illegal character */
			printf("%c is not a valid hex character.\n",*str);
			return -1;
		}
		if (debugsw>5) printf("Val now %d.\n",val);
	}
	return val;
}

void
dump_int(void)
{		/* dump info about integer */
	printf("int1 is at address %x.\n", &int1);
	printf("Value of int1 is %d.\n", int1);
	printf("int2 is at address %x.\n", &int2);
	printf("Value of int2 is %d.\n", int2);
	printf("Address of intp is %x, contents is %x\n", &intp, intp);
	printf("Value of integer pointed to by intp is %d.\n", *intp);
}

void
dump_str(void)
{		/* dump info on string variables */
	printf("Work string is at address %x.\n", &str);
	printf("Work string contains >>%s<<\n", str);
	printf("String Pointer points to %x.\n", strp);
	printf("Character pointed to by string pointer %x (%c)\n", *strp, *strp);
}

