/*
 *				**** DO SEARCH ****
 *
 * This file contains routines to demonstrate various table searches
 *
 */

#include "demo.h"

void do_srch(char *);
void first_srch(char *);
void last_srch(char *);
void unique_srch(char *);
void num_srch(char *);
void all_srch(char *);
void chg_srch(char *);
void part_srch(char *);

void
do_srch(char *who)
{		/* experiment with searching tables */
	char	buf[BUFSIZ];	/* input buffer */
	char	*p;				/* scan pointer */
	int		opt;			/* menu option */

	printf("Hello, %s.  Welcome to the Searching Routines.\n", who);

	if (persons == 0)
	{		/* list is empty */
		printf("There are no entries in the lists to be searched.\n");
		printf("You may want to go back to the main menu.\n");
		printf("And select to `Structures' option.\n");
	} else
	{
		printf("There are %d persons in the list.\n", persons);
	}

	do
	{		/* display menu & get option */
		printf("Searching Options are:\n\n");
		printf("\tQ\tReturn to main menu.\n\n");
		printf("\tF\tFind first match in list.\n");
		printf("\tL\tFind last match in list.\n");
		printf("\tU\tFind unique match.\n");
		printf("\tN\tDisplay number of matches.\n");
		printf("\tA\tDisplay all matches.\n");
        printf("\tP\tPartial match in list.\n");
		printf("\tC\tChange all matching entries.\n");
		printf("\n\t?\tDisplay help message.\n");

		printf("\nEnter Option: ");
		
		gets(buf);
		for (p=buf; *p == ' '; p++);	/* find non-blank */
		switch (opt = *p)
		{	/* branch on value of opt */
			case 'Q':	/* quit */
			case 'q':
				opt = 0;	/* signal done looping */
				break;	/* nothing to do */

			case 'd':	/* display list */
			case 'D':
				list_struct(who);
				break;

			case 'f':	/* find first match */
			case 'F':
				first_srch(who);
				break;

			case 'l':	/* find last match */
			case 'L':
				last_srch(who);
				break;

			case 'u':	/* find unique match */
			case 'U':
				unique_srch(who);
				break;

			case 'n':	/* find number of matches */
			case 'N':
				num_srch(who);
				break;

			case 'a':	/* find all matches */
			case 'A':
				all_srch(who);
				break;

			case 'c':	/* change matches */
			case 'C':
				chg_srch(who);
				break;

            case 'p':	/* partial match */
            case 'P':
				part_srch(who);
                break;

			case '?':	/* help match */
				help("search");
				break;

			default:
				printf("Sorry, %s, but option %s is invalid.\n", who, buf);
		}
	} while (opt);

	if (debugsw > 0) printf("Leaving do_search.\n");
}

void
first_srch(char *who)
{		/* find first match in list */
	char	buf[BUFSIZ];	/* input field */
	int		i;				/* loop variable */

	if (debugsw) printf("Entering first_srch, argument >>%s<<\n",  who);

	printf("Enter last name of user to search for: ");
	gets(buf);				/* get name */

	/* search until you find a match (or end of list) */
	for (i=0; i < persons && strcmp(buf,people[i].p_lname); i++);

	if (i < persons)
	{		/* entry found - report result */
		printf("%s first found at entry %d\n", buf, i);
		list_ent(i);	/* list entry */
	} else printf("Last name %s not found.\n", buf);
}

void
last_srch(char *who)
{		/* find last match in list */
	char	buf[BUFSIZ];	/* input buffer */
	int		i, match=-1;		/* loop var & last match */

	if (debugsw) printf("Entering last_srch, argument >>%s<<\n",  who);

	printf("Enter last name of user to search for: ");
	gets(buf);				/* get name */

	/* find the last match */
	/* we could also search this list in the reverse order */
	/* then it would work the same as the first match */

	for (i=0; i < persons; i++)
		if (strcmp(buf,people[i].p_lname) == 0) match = i;

	if (match >= 0)
	{		/* entry found - report match */
		printf("%s last found at entry %d\n", buf, match);
		list_ent(match);
	} else printf("Last name %s not found.\n", buf);
}

void
unique_srch(char *who)
{		/* find unique match (duplicate is an error) */
	char	buf[BUFSIZ];
	int		i, match=-1;	/* loop variable & match flag */

	if (debugsw) printf("Entering unique_srch, argument >>%s<<\n",  who);


	printf("Enter last name of user to search for: ");
	gets(buf);				/* get name */

	for (i=0; i < persons; i++)
	{		/* loop looking for a match */
		if (strcmp(buf,people[i].p_lname) == 0)
		{		/* match found - make sure it is unique */
			if (match != -1)
			{		/* not first match - complain */
				printf("*** Error **** %s also found at %d\n", buf, i);
			} else
			{		/* first time match */
				printf("Last name %s found at entry %d\n", buf, i);
				list_ent(i);	/* display entry */
				match = i;		/* save location */
			}
		}
	}

	if (match == -1)
		printf("Last name %s not found.\n", buf);
}

void
num_srch(char *who)
{		/* find number of matches */
	char	buf[BUFSIZ];
	int		i, matches=0;			/* loop variable & counter */


	if (debugsw) printf("Entering num_srch, argument >>%s<<\n",  who);

	printf("Enter last name of user to search for: ");
	gets(buf);				/* get name */

	for (i=0; i < persons; i++)
		if (strcmp(buf,people[i].p_lname) == 0) matches++;

	printf("Last name %s found %d times in list.\n", buf, matches);
}

void
all_srch(char *who)
{		/* list all entries that match */
	char	buf[BUFSIZ], junk[BUFSIZ];
	int		i, matches=0;	/* loop variable & match counter */

	if (debugsw) printf("Entering all_srch, argument >>%s<<\n",  who);

	printf("Enter last name of user to search for: ");
	gets(buf);				/* get name */

	for (i=0; i < persons; i++)
	{		/* loop thru table */
		if (strcmp(buf,people[i].p_lname) == 0)
		{		/* match found - display entry */
			printf("Last name %s found at entry %d\n", buf, i);
			list_ent(i);
            matches++;
			printf("Press ENTER to continue: ");
			gets(junk);
		}
	}
    printf("%d matches found.\n", matches);
}

void
chg_srch(char *who)
{		/* change all entries that match */
	char	buf[BUFSIZ];
	int		i, matches=0;	/* loop variable & match flag */

	if (debugsw) printf("Entering chg_srch, argument >>%s<<\n",  who);

	printf("Enter last name of user to search for: ");
	gets(buf);				/* get name */

	for (i=0; i < persons; i++)
	{
		if (strcmp(buf,people[i].p_lname) == 0)
		{
			ent_struct(&people[i]);
			matches++;
		}
	}
	printf("%d matches found.\n", matches);
}

void
part_srch(char *who)
{		/* perform partial match & save matches in table */
	char	buf[BUFSIZ];	/* input buffer */
    int		i, matches=0;	/* loop variable & match counter */
    int		mtbl[100];		/* table for matches */
    int		len;			/* length of string entered */

    if (debugsw) printf("Entering part_srch, argument >>%s<<\n", who);

    printf("Enter (partial) last name to search for: ");
    gets(buf);
    len = strlen(buf);		/* get length of response for compare */

	for (i=0; i < persons; i++)
    {		/* loop thru structure of persons */
		if (strncmp(buf,people[i].p_lname,len)) continue;	/* no match */
        mtbl[matches++] = i;	/* save match location */
    }

   	printf("%d matches found:\n", matches);	/* how many matches */

	if (matches)
    {		/* at least one match - display them */
    	printf("No.\tIndex\tP ID\n");
    	for (i=0; i < matches; i++)
        {
        	printf("%d.\t%d\t%s\n", i+1, mtbl[i], people[mtbl[i]].p_id);
		}

        printf("Enter no. to display (1-%d): ", matches);
        i = atoi(gets(buf));	/* get answer as a number */
        if (i > 0 && i <= matches) list_ent(mtbl[i-1]);
        else printf("Invalid number.\n");
    }
}
