/*
 *				**** HELP ****
 *
 * This routine provides a help facility for the demo program.
 *
 * It uses the passed parameter string as an index into the help
 *	file, and then displays the contents of that section of the
 *	file.
 *
 * Sections are indicated by three asterisks followed by the label
 *	(e.g., ***DEMO)
 *
 * Page breaks are indicated by the line "---MORE"
 *
 * If the user enters 'q', the help routine will quit.
 *
 * Notice that this is a stand alone routine - nothing depends on the
 *	actual demo program.  This routine can be used by other programs.
 *
 */

#include <stdio.h>
#include <string.h>

#define	MORE	"---MORE"

static FILE	*fp = NULL;		/* help file pointer */

void
help_init(char *fname)
{		/* open help file */
	fp = fopen(fname,"r");	/* open for input */
}

void
help_close(void)
{		/* close the help file */
	fclose(fp);
}

void
help(char *subj)
{		/* display help section of file */
	char	buf[BUFSIZ];		/* input buffer */
	char	Subj[80];			/* subject line */
	int		Page = 0;			/* what help page we are on */
	char	*p;					/* pointer for scanning input line */

	strcpy(Subj,"***");			/* prefix for header */
	strcat(Subj,subj);			/* build search key */
	
	rewind(fp);					/* start at top of file */
	
	while (fgets(buf,sizeof(buf),fp) != NULL)
	{		/* valid line read */
		for (p = buf; *p && *p != '\r' && *p != '\n'; p++);	/* end of text */
		*p = '\0';				/* strip CR-LF sequence */
		/* see if we found the section yet */
		if (Page == 0 && stricmp(buf,Subj)) continue;
		/* see if past section */
		if (strncmp(buf,"***",3) == 0)
		{
			if (Page > 0) break;		/* end of section */
			/* first time for subject */
			Page = 1;					/* current page number */
			continue;					/* don't print header line */
		}

		if (strcmp(buf,MORE) == 0)
		{		/* end of page */
			printf("Press Enter for next page: ");
			gets(buf);		/* read from user */
         if (*buf == 'q' || *buf == 'Q') break;
			Page++;			/* next page */
			continue;		/* on to next page */
		}

			/* regular line, display it */
		puts(buf);			/* output it to the screen */
	}
	
	if (Page == 0)
	{		/* not found */
		fprintf(stderr,
			"Internal error!  Help subject >>%s<< not found.\n", subj);
	} else
	{		/* was found, keep it from scrolling */
		printf("Press Enter to continue: ");
		gets(buf);
	}
}

#ifdef EBUG
/*
 * When you set a define of EBUG, the compiler will include the following
 * lines.  This is useful when debugging a routine such as this.
 *
 */

 int
main(void)
{
	char buf[BUFSIZ];

	help_init("demo.hlp");

	do
	{
		printf("Enter help topic, or just Return to exit: ");
		gets(buf);
		if (*buf) help(buf);
	} while (*buf);

	help_close();
}
#endif
