/*
 *				**** ARRAY ****
 *
 * This file contains the routines related to arrays
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "datademo.h"

struct Entry Array[MAX_ENTS];	/* array of entries */
int num_ents = 0;					/* number of entries in array */

/* function prototypes */
void add_array(void);
void delete_array(void);
void list_array(void);
void search_array(void);
void display_array(int ent);

int
do_array(void)
{
	char buf[BUFSIZ];

   while (1)
   {
   	printf("\nOptions are:\n");
      printf("A. Add entries to array\n");
      printf("D. Delete entries from array\n");
      printf("L. List array\n");
      printf("S. Search array\n");
      printf("M. Return to Main Menu\n");
      printf("Q. Quit\n");
      printf("Enter option: ");
      gets(buf);
      switch(buf[0])
      {
      case 'a':
      case 'A':
      	add_array();
         break;
      case 'd':
      case 'D':
      	delete_array();
         break;
      case 'l':
      case 'L':
      	list_array();
         break;
      case 's':
      case 'S':
      	search_array();
         break;
      case 'm':
      case 'M':
      	return 1;	/* keep running */

      case 'q':
      case 'Q':
      	return 0;	/* don't keep running */

      default:
      	printf("Invalid option - try again!\n");
      }
   }
}

void
add_array(void)
{ /* add an entry to the array */
	char buf[BUFSIZ];
   int key;

   printf("There are %d entries currently in use\n", num_ents);

   /* make sure array isn't full */
   if (num_ents >= MAX_ENTS)
   {
   	printf("Array is full.  Delete some entries first\n");
      return;
   }

   /* ready to add to end of array */
   printf("Enter key: ");
   gets(buf);
   key = atoi(buf);		/* set key */
   do
   {
	   printf("Enter description: ");
	   gets(buf);
	   if (strlen(buf) > MAX_LEN)
      {
      	printf("Name too long.  Try again? (y or n): ");
         gets(buf);
         if (buf[0] == 'n') return;
      }
   } while(strlen(buf) >= MAX_LEN);

   printf("Adding new entry in location %d\n", num_ents);
   Array[num_ents].key = key;
   Array[num_ents].use = USE_YES;
   strcpy(Array[num_ents].desc, buf);
   num_ents++;		/* one more entry in array */
   printf("There are now %d entries in use\n", num_ents);
}

void
delete_array(void)
{
	char buf[BUFSIZ];
   int i, ent;

   printf("There are currently %d entries in use\n", num_ents);

   printf("Enter entry to delete: ");
   gets(buf);
   ent = atoi(buf);
   if (ent < 0 || ent >= num_ents)
   { /* out of range */
   	printf("There is no entry # %d in use.  Try again!\n");
      return;
   }

   printf("Entry to be removed:\n");
   display_array(ent);
   printf("OK? (y or n): ");
   gets(buf);
   if (buf[0] != 'y') return;		/* don't remove */

   do
   {
	   printf("Method to fill in the empty space:\n");
	   printf("1. Move last entry into deleted entry\n");
	   printf("2. Shift lower entries up one space\n");
      printf("Enter option: ");
      gets(buf);
   } while (buf[0] != '1' && buf[0] != '2');

   /* delete entry by moving something over the deleted entry */
   if (buf[0] == '1')
   { /* move last entry into now-empty space */
   	printf("Moving entry %d to location %d\n", num_ents-1,
      	ent);
   	Array[ent].key = Array[num_ents - 1].key;
      strcpy(Array[ent].desc, Array[num_ents - 1].desc);
   }
   else
   { /* shift everything up one level */
   	printf("Shifting entries %d thru %d up one slot\n",
      	ent+1, num_ents-1);
   	for (i=ent; i < num_ents-1; i++)
      { /* shift everything up a level */
      	Array[i].key = Array[i+1].key;
			strcpy(Array[i].desc, Array[i+1].desc);
      }
   }

	/* deleted entry overwritten -- decrement counter */
   num_ents--;		/* one less entry in array */
   printf("There are now %d entries in use\n", num_ents);
}

void
list_array(void)
{
	int i;

   printf("%d entries in table ...\n", num_ents);

   for (i=0; i < num_ents; i++) display_array(i);
}

void
display_array(int ent)
{
	printf("Entry %d, key %d, desc: %s\n",
   	ent, Array[ent].key, Array[ent].desc);
}

void
search_array(void)
{
	char buf[BUFSIZ];
   int key, i, num_matches = 0;

   printf("Enter key to search for: ");
   gets(buf);
   key = atoi(buf);

   printf("Searching for key: %d\n", key);

   for (i=0; i < num_ents; i++)
   {
   	if (key == Array[i].key)
      { /* key match found */
      	display_array(i);
         num_matches++;
      }
   }
   printf("%d matches found\n", num_matches);
}

void
init_array(void)
{
	int i;

   /*
    * Initialize Array
    *
    * set number of entries to zero
    * clear all entries in the array
    */
	num_ents = 0;		/* how many entries in array */
	for (i=0; i < MAX_ENTS; i++)
   { /* clear array entries */
   	Array[i].use = USE_NOT;		/* not in use */
      Array[i].key = 0;					/* no key */
      Array[i].desc[0] = '\0';		/* clear description */
   }
}

