/*
 *				**** QUEUE ****
 *
 * This file contains the routines related to queues
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "datademo.h"

struct Queue Queue;		/* queue of entries */

/* function prototypes */
void add_queue(void);
void delete_queue(void);
void list_queue(void);
void search_queue(void);
void display_queue(int ent);
int next_entry(int ent);		/* entry in front of this one */
int prev_entry(int ent);		/* entry behind this one */

int
do_queue(void)
{
	char buf[BUFSIZ];

   while (1)
   {
   	printf("\nOptions are:\n");
      printf("A. Add entries to queue\n");
      printf("D. Delete entries from queue\n");
      printf("L. List queue\n");
      printf("S. Search queue\n");
      printf("M. Return to Main Menu\n");
      printf("Q. Quit\n");
      printf("Enter option: ");
      gets(buf);
      switch(buf[0])
      {
      case 'a':
      case 'A':
      	add_queue();
         break;
      case 'd':
      case 'D':
      	delete_queue();
         break;
      case 'l':
      case 'L':
      	list_queue();
         break;
      case 's':
      case 'S':
      	search_queue();
         break;
      case 'm':
      case 'M':
      	return 1;	/* keep running */

      case 'q':
      case 'Q':
      	return 0;	/* don't keep running */

      default:
      	printf("Invalid option - try again!\n");
      }
   }
}

void
add_queue(void)
{ /* add an entry to the tail of the queue */
	char buf[BUFSIZ];
   int key, newloc;

   printf("There are %d entries currently in use\n", Queue.num_ents);

   /* make sure queue isn't full */
   if (Queue.num_ents >= MAX_ENTS)
   {
   	printf("Queue is full.  Delete some entries first\n");
      return;
   }

   /* ready to add to end of queue */
   printf("Enter key: ");
   gets(buf);
   key = atoi(buf);		/* set key */
   do
   {
	   printf("Enter description: ");
	   gets(buf);
	   if (strlen(buf) > MAX_LEN)
      {
      	printf("Name too long.  Try again? (y or n): ");
         gets(buf);
         if (buf[0] == 'n') return;
      }
   } while(strlen(buf) >= MAX_LEN);

   newloc = next_entry(Queue.tail);
   printf("Adding new entry in location %d\n", newloc);
   Queue.Entry[newloc].key = key;
   Queue.Entry[newloc].use = USE_YES;
   strcpy(Queue.Entry[newloc].desc, buf);

   /* add entry to tail of queue */
   Queue.tail = newloc;
   Queue.num_ents++;		/* one more entry in queue */
   printf("There are now %d entries in use\n", Queue.num_ents);
}

void
delete_queue(void)
{ /* delete entry from head of queue */
	char buf[BUFSIZ];
   int ent;

   printf("There are currently %d entries in use\n", Queue.num_ents);
	if (Queue.num_ents <= 0) return;	/* queue empty - nothing to delete */

   ent = Queue.head;		/* entry on top of queue */
   printf("Entry to be removed:\n");
   display_queue(ent);
   printf("OK? (y or n): ");
   gets(buf);
   if (buf[0] != 'y') return;		/* don't remove */

   /* delete entry from head */
   Queue.head = next_entry(Queue.head);	/* second entry now at head */

   Queue.num_ents--;		/* one less entry in queue */
   printf("There are now %d entries in use\n", Queue.num_ents);

   /* if queue is empty - reset head & tail to zero */
   if (Queue.num_ents == 0)
   {
   	Queue.head = Queue.tail = 0;
   }
}

void
list_queue(void)
{
	int i, n=0;
   char buf[BUFSIZ];

   printf("%d entries in table, head=%d, tail=%d ...\n",
   	Queue.num_ents, Queue.head, Queue.tail);

   for (i=Queue.head; n < Queue.num_ents; i = next_entry(i))
   {
   	display_queue(i);
      if ((++n % 20) == 19)
      {
      	printf("Press ENTER to continue: ");
         gets(buf);
      }
   }
}

void
display_queue(int ent)
{
	printf("Entry %d, key %d, desc: %s\n",
   	ent, Queue.Entry[ent].key, Queue.Entry[ent].desc);
}

void
search_queue(void)
{
	char buf[BUFSIZ];
   int key, i, num_matches = 0;

   printf("Enter key to search for: ");
   gets(buf);
   key = atoi(buf);

   printf("Searching for key: %d\n", key);

   for (i=Queue.head; i != Queue.tail; i=next_entry(i))
   {
   	if (key == Queue.Entry[i].key)
      { /* key match found */
      	display_queue(i);
         num_matches++;
      }
   }
   printf("%d matches found\n", num_matches);
}

void
init_queue(void)
{
	int i;

   /*
    * Initialize Queue
    *
    * set number of entries to zero
    * clear all entries in the queue
    */
	Queue.num_ents = 0;		/* how many entries in queue */
   Queue.head = 0;
   Queue.tail = 0;
	for (i=0; i < MAX_ENTS; i++)
   { /* clear queue entries */
   	Queue.Entry[i].use = USE_NOT;		/* not in use */
      Queue.Entry[i].key = 0;					/* no key */
      Queue.Entry[i].desc[0] = '\0';		/* clear description */
   }
}

int
next_entry(int ent)
{ /* return the entry behind ent */
	if (Queue.num_ents <= 0) return 0;	/* empty - next entry is first entry */
   if (ent > (MAX_ENTS-1)) return 0;	/* end of queue - wrap around */
   return (ent+1);				/* normal entry */
}

int
prev_entry(int ent)
{ /* return entry in front of ent */
	if (ent > 0) return (ent-1);
   return (MAX_ENTS-1);
}

