/*
 *				**** EVENT ****
 *
 * This program is an example of a simple event driven
 * program.  A real event driven program would be much
 * more complex, and might even have several programs
 * running at the same time and 'talking' to each other
 *
 * THIS PROGRAM WILL ONLY RUN UNDER MS-DOS
 * It uses the ms-dos call 'getch' which will get a
 * single character from the console
 */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <ctype.h>
#include "fkeys.h"

int get_key(void);
void process_key(int ch);
void reg_key(int ch);
void spec_key(int ch);
void do_help(void);
void help_fkeys(void);

/*
 * Global Variables
 */

int running = 1;	/* switch indicating we are running */
int cnt=0;			/* times through loop */

/* input buffer */
char buf[BUFSIZ];
int bufptr = 0;	/* location of next position in buffer */

main()
{
	int ch;		/* keyboard character */

   puts("Press F1 for help");
   
 	while (running)
   { /* loop until done */
   	ch = get_key();
      if (ch > 0)
      {
			process_key(ch);
      }
   }
}

/*
 * Determine if a key has been pressed and return
 * its code.  If no key, return negative
 */
int
get_key(void)
{
	int c;

   /* if no key is present, return -1 */
   if (!kbhit()) return NOKEY;	/* no key present */

   /*
    * A key was pressed - get scancode
    * If a special key, return the next scancode
    * shiften into the second byte
    */
	c = getch();
	if (c == 0)
	{ /* special key - multiply scan code by X'100' */
		c = getch();	/* special key scancode */
		c <<= 8;			/* shift (multiply) */
	}

	return c; /* return key code */
}

void
process_key(int ch)
{ /* process the pressed key */
   if (isprint(ch))
   { /* regular key */
   	reg_key(ch);
   }
   else
   {
   	spec_key(ch);
   }
}

void
reg_key(int ch)
{ /* insert a character into the buffer */
	/* make sure there is room in buffer */
   if (bufptr > BUFSIZ-1)
   { /* error - buffer full */
   	puts("Buffer full - use backspace or ENTER");
      return;	/* don't add this character */
   }
   buf[bufptr] = ch;
   bufptr++;
}

void
spec_key(int ch)
{
	switch(ch)
   { /* process individual special key */
   case BS: /* backspace - remove previous char */
   	if (bufptr > 0)
      { /* characters in buffer - remove last one */
      	bufptr--;	/* back up one character */
      }
      else puts("The buffer is empty!");
      break;
   case ENTER: /* end of line */
   	buf[bufptr] = '\0'; /* terminate buffer */
      printf("Line entered >>%s<<\n", buf);
      bufptr = 0;		/* reset counter to zero */
      break;
   case ESC: running = 0; break;
   case F1: do_help(); break;
   case F2: help_fkeys(); break;
   case F5:	/* display line so far */
   	buf[bufptr] = '\0';	/* terminate for display */
      printf("Line so far >>%s<<\n", buf);
   	break;
   default:
		printf("Special key (%d) not supported\n", ch);
   }
}

void
do_help(void)
{ /* display a help message */
	puts("Enter a line of text, then press ENTER.");
   puts("Nothing will be displayed until you press ENTER");
   puts("Press F2 for a list of function keys");
}

void
help_fkeys(void)
{
	puts("F1 - Help");
   puts("F2 - This list");
   puts("F5 - Display line so far");
   puts("ENTER - finish line");
   puts("ESC - Exit");
}

