/*
 *				**** EVENT ****
 *
 * This program is an example of a simple event driven
 * program.  A real event driven program would be much
 * more complex, and might even have several programs
 * running at the same time and 'talking' to each other
 *
 * THIS PROGRAM WILL ONLY RUN UNDER MS-DOS
 * It uses the ms-dos call 'getch' which will get a
 * single character from the console
 */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <ctype.h>
#include "fkeys.h"

/*
 * Definitions
 */
#define BUFFER_SIZE 200	/* size of input buffer */
#define MAX_BUFS 5	/* max. number of buffers */

/* input buffer structure */
struct InputBuffer {
	char buf[BUFFER_SIZE]; /* actual buffer */
	int bufptr;		 /* location of next position in buffer */
};

int get_key(void);
void do_init(void);
void process_key(int ch);
void reg_key(int ch);
void spec_key(int ch);
void do_help(void);
void help_fkeys(void);
void display_status(void);

/*
 * Global Variables
 */

int running = 1;	/* switch indicating we are running */
int cnt=0;			/* times through loop */

struct InputBuffer BufTbl[MAX_BUFS];
int curr_buf = 0;	/* current input buffer */

main()
{
	int ch;		/* keyboard character */

   do_init();	/* initialize things */

   puts("Press F1 for help");

 	while (running)
   { /* loop until done */
   	ch = get_key();
      if (ch > 0)
      {
			process_key(ch);
      }
   }
}

void
do_init(void)
{ /* initialize whatever is necessary */
	int i;

   /* clear the input buffers */
   for (i=0; i < MAX_BUFS; i++)
   {
   	BufTbl[i].bufptr = 0;
      BufTbl[i].buf[0] = '\0';
   }
   curr_buf = 0;		/* start with buffer zero */
}

/*
 * Determine if a key has been pressed and return
 * its code.  If no key, return negative
 */
int
get_key(void)
{
	int c;

   /* if no key is present, return -1 */
   if (!kbhit()) return NOKEY;	/* no key present */

   /*
    * A key was pressed - get scancode
    * If a special key, return the next scancode
    * shiften into the second byte
    */
	c = getch();
	if (c == 0)
	{ /* special key - multiply scan code by X'100' */
		c = getch();	/* special key scancode */
		c <<= 8;			/* shift (multiply) */
	}

	return c; /* return key code */
}

void
process_key(int ch)
{ /* process the pressed key */
   if (isprint(ch))
   { /* regular key */
   	reg_key(ch);
   }
   else
   {
   	spec_key(ch);
   }
}

void
reg_key(int ch)
{ /* insert a character into the buffer */
	/* make sure there is room in buffer */
   if (BufTbl[curr_buf].bufptr > BUFFER_SIZE-1)
   { /* error - buffer full */
   	puts("Buffer full - use backspace or ENTER");
      return;	/* don't add this character */
   }
   BufTbl[curr_buf].buf[BufTbl[curr_buf].bufptr] = ch;
   BufTbl[curr_buf].bufptr++;
}

void
spec_key(int ch)
{
	switch(ch)
   { /* process individual special key */
   case BS: /* backspace - remove previous char */
   	if (BufTbl[curr_buf].bufptr > 0)
      { /* characters in buffer - remove last one */
      	BufTbl[curr_buf].bufptr--;	/* back up one character */
      }
      else puts("The buffer is empty!");
      break;
   case TAB: /* tab - move to next buffer */
   	curr_buf++;		/* increment current buffer */
      /* wrap around to first buffer if at end */
      if (curr_buf >= MAX_BUFS) curr_buf = 0;
      printf("Now using buffer %d\n", curr_buf);
      break;
   case SHIFT_TAB: /* shift tab - move to prev buffer */
      curr_buf--;		/* decrement current buffer */
      /* handle wrap around to last buffer */
      if (curr_buf < 0) curr_buf = MAX_BUFS - 1;
      printf("Now using buffer %d\n", curr_buf);
      break;
   case ENTER: /* end of line */
   	BufTbl[curr_buf].buf[BufTbl[curr_buf].bufptr] = '\0'; /* terminate buffer */
      printf("Line entered >>%s<<\n", BufTbl[curr_buf].buf);
      BufTbl[curr_buf].bufptr = 0;		/* reset counter to zero */
      break;
   case ESC: running = 0; break;
   case F1: do_help(); break;
   case F2: help_fkeys(); break;
   case F5:	/* display line so far */
   	BufTbl[curr_buf].buf[BufTbl[curr_buf].bufptr] = '\0';	/* terminate for display */
      printf("Line so far >>%s<<\n", BufTbl[curr_buf].buf);
   	break;
   case F6: display_status(); break;	/* current status */
   case F10: /* reset everything */
   	do_init();
      puts("System reset");
      break;
   default:
		printf("Special key (%d) not supported\n", ch);
   }
}

void
do_help(void)
{ /* display a help message */
	puts("Enter a line of text, then press ENTER.");
   puts("Nothing will be displayed until you press ENTER");
   puts("Press F2 for a list of function keys");
}

void
help_fkeys(void)
{
	puts("F1 - Help");
   puts("F2 - This list");
   puts("F5 - Display line so far");
   puts("F6 - Display Status");
   puts("F10  Reset");
   puts("ENTER - finish line");
   puts("ESC - Exit");
}

void
display_status(void)
{
	int i;

   printf("Current buffer is %d\n", curr_buf);

   printf("Current buffer status:\n");
   for (i=0; i < MAX_BUFS; i++)
   {
   	/* add terminating character to input buffer */
   	BufTbl[i].buf[BufTbl[i].bufptr] = '\0';
   	printf("Buffer %d, bufptr=%d >>%s<<\n", i,
      		BufTbl[i].bufptr, BufTbl[i].buf);
   }
}

