/*
 *				**** PARSE ****
 *
 * This program demonstrates one way to parse text
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* table used to convert month name to/from number */
char *Month[] = {
   "January", "February", "March", "April", "May", "June",
   "July", "August", "September", "October", "November",
   "December", NULL};

/* table containing max. days per month */
int MaxDay[] = {
	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

/* default areacode */
int DefaultAreaCode = 610;

void help_strtok();
void use_strtok(void);
void help_strtok(void);
void parse_manually(void);
void parse_words(void);
void parse_date(void);
void parse_phone(void);
void parse_help(void);

int
main(void)
{
	char buf[BUFSIZ];
   int opt;

   do
   {
   	printf("\nOptions are:\n");
      printf(" 1. Using strtok\n");
      printf(" 2. Parsing it yourself\n");
      printf(" 9. Exit\n");
		printf("\nEnter option: ");
      gets(buf);
      opt = atoi(buf);
      switch(opt)
      {
      case 1: use_strtok(); break;
      case 2: parse_manually(); break;
      case 9: printf("Bye!\n"); break;
      default:
      	printf("Unknown option (%s)\n", buf);
      }
   } while (opt != 9);

   return 0;
}

void
use_strtok(void)
{
	char buf[BUFSIZ], str[BUFSIZ], delim[BUFSIZ], *p;
   char *flds[20];		/* fields */
   int nflds=0;

   printf("Do you want help? (y or n): ");
   gets(buf);
   if (buf[0] == 'y')
   {
	   help_strtok();
      printf("Press ENTER to continue: ");
      gets(buf);
   }

   printf("Enter string to parse: ");
   gets(str);	/* string to parse */
	strcpy(delim, " \t");	/* default delimeters */

   p = str;		/* first time, we pass the string */
   while (1)
   {
   	printf("Enter delimeter(s) for first field [%s]: ",
      			delim);
      gets(buf);
      if (buf[0] != '\0')
      {
      	strcpy(delim, buf);
      }
		flds[nflds] = strtok(p, delim);	/* parse next field */
      if (flds[nflds] == NULL)
      { /* no more tokens */
      	printf("No more tokens remain\n");
         break;
      }

      /* we have another token */
      printf("Field %d is >>%s<<\n", nflds, flds[nflds]);
      nflds++;		/* increment number of tokens */
      p = NULL;	/* remaining times, pass NULL */
   }
}

void
help_strtok()
{
   printf("This routine will allow you to parse strings into fields.\n");
   printf("Fields are separated by delimeters.\n");
   printf("Each field can have a different delimeter\n");
   printf("For example, a date field would use slashes: 9/12/97\n");
   printf("A phone number might use more than one delimeter: (610_555-1212\n");
   printf("\n");
   printf("When asked for a string, enter a phrase or sentence\n");
   printf("When asked for a delimeter, you can press ENTER for spaces and tabs\n");
   printf("or you can enter character(s) might terminate the next word.\n");
}

void
parse_manually(void)
{
	char buf[BUFSIZ];
	int opt;

   while(1)
   {
	   printf("\nOptions are:\n");
      printf("1. Parse line into words\n");
      printf("2. Parse date\n");
      printf("3. Parse phone number\n");
	   printf("8. Help\n");
		printf("9. Back to main menu\n");
	   printf("Enter option: ");
	   gets(buf);
      opt = atoi(buf);
	   switch(opt)
	   {
      case 1: parse_words(); break;
      case 2: parse_date(); break;
      case 3: parse_phone(); break;
	   case 8: parse_help(); break;
      case 9: return;
	   default:
	   	printf("Invalid option (%s) - try again\n", buf);
	   }
      printf("Press ENTER to continue: ");
      gets(buf);
   }
}

void
parse_words(void)
{ /* parse a line into words */
	char buf[BUFSIZ], out[BUFSIZ];
   int i=0, j;

   printf("Enter line to parse into words: ");
   gets(buf);

   /*
    * divide the line up into words
    *
    * The technique we use is quite simple:
    * find the first alphabetic character (start of a word)
    * find the first non-alphabetic character (end of word)
    * continue until the end of the string
    */
	i = 0;
    while (buf[i] != '\0')
    {
	    /* find start of word */
	   for (; buf[i] != '\0' && !isalpha(buf[i]); i++);
		if (buf[i] == '\0') break;	/* end of string found */

      /* copy the word to 'out' */
      for (j = 0; isalpha(buf[i]); i++)
      {
      	out[j++] = buf[i];
      }
      /* terminate word string */
      out[j] = '\0';
      printf("Word found: (%s)\n", out);
    }
    printf("End of string\n");
}

void
parse_date(void)
{ /* parse date field mm/dd/yy */
	char buf[BUFSIZ];
   int mon, day, year;
   int beg;

   printf("Enter date to parse (mm/dd/yy): ");
   gets(buf);

   /* find start of the actual date (month) */
   for (beg = 0; buf[beg] != '\0' && isspace(buf[beg]); beg++);

   /*
    * when done, beg contains location of first non-space
    * this should be a numeric digit
    */

	if (!isdigit(buf[beg]))
   {
   	printf("Invalid date (doesn't start with digit)\n");
      return;
   }

   /*
    * the first character of the date is a digit
    *
    * the month should be one or two digits,
    * followed by a slash
    *
    * look for the slash in either the next position
    * or the one after (for two digit month)
    */
   mon = atoi(buf+beg);	/* convert month to number */
   if (mon < 1 || mon > 12)
   {
   	printf("Invalid date (invalid month)\n");
      return;
   }
	if (buf[beg+1] == '/')
   { /* one digit month */
      mon = atoi(buf+beg);	/* convert month to number */
      beg += 2;				/* start of day field */
   }
   else if (buf[beg+2] == '/')
   { /* two digit month number */
   	if (! isdigit(buf[beg+1]))
      {
      	printf("Invalid date: second month char. not a digit\n");
         return;
      }
		beg += 3;			/* start of day field */
   }
   else
   {
   	printf("Invalid date (missing / after month)\n");
      return;
   }

   /*
    * we now have the month of this date as a number
    *
    * look for day in much the same way as month
    * the day can be a one or two digit number
    */
    day = atoi(buf+beg);		/* convert day to number */
    if (day < 1 || day > MaxDay[mon-1])
    { /* returns zero if okay */
    	printf("Invalid date (invalid day)\n");
      return;
    }
    if (!isdigit(buf[beg]))
    {
    	printf("Invalid date (first char. of day not a digit)\n");
      return;
    }
	if (buf[beg+1] == '/')
   { /* one digit day */
   	beg += 2;		/* first digit of year */
   }
   else if (buf[beg+2] == '/')
   { /* two digit day */
   	if (!isdigit(buf[beg+1]))
      {
      	printf("Invalid date (second char of day not a digit)\n");
         return;
      }
   	beg += 3;
   }
   else
   {
   	printf("Invalid date (missing / after day)\n");
      return;
   }

   /*
    * we now have the month and day as numbers
    * finally, we convert the year to a number
    */
    year = atoi(buf + beg);
    if (year < 1900) year += 1900;	/* convert to 4 digits */

    /* now display the date */
    printf("Date is %s %d, %d\n", Month[mon-1], day, year);
}

void
parse_phone(void)
{
	char buf[BUFSIZ], out[BUFSIZ];
   int i, j, areacode, exchange, number;

   printf("Enter phone number to parse: ");
   gets(buf);

	for (i=j=0; buf[i] != '\0'; i++)
   { /* loop thru input line and copy digits to output */
   	if (isdigit(buf[i])) out[j++] = buf[i];
   }

   /* now terminate phone number string */
   out[j] = '\0';

   /*
    * we now examine what we have copied
    * and try to guess what format we have been given
    *
    * we look at the length of the string
    * if the length is 7, then we have XXX-NNNN
    * if the length is 10, we have AAA-XXX-NNNN
    * if the length is 11, we have 1-AAA-XXX-NNNN
    * anything else, and we have an invalid phone number
    *
    * (where AAA is area code, XXX is exchange,
    * and NNNN is the number)
    */

    switch(strlen(out))
    {
    case 7:	/* XXX-NNNN */
    	number = atoi(out+3);	/* convert number */
      out[3] = '\0';		/* terminate exchange */
      exchange = atoi(out);
      areacode=DefaultAreaCode;	/* no area code */
      break;
	case 10: /* AAA-XXX-NNNN */
		number = atoi(out+6);	/* convert number */
      out[6] = '\0'; /* terminate exchange */
      exchange = atoi(out+3);	/* get exchange */
      out[3] = '\0';	/* terminate area code */
      areacode = atoi(out);
      break;
   case 11: /* 1-AAA-XXX-NNNN */
   	number = atoi(out+7);	/* get number */
      out[7] = '\0';	/* terminate exchange */
      exchange = atoi(out + 4);
      out[4] = '\0';
      areacode = atoi(out+1);
      if (out[0] != '1')
      {
      	printf("Invalid phone number 1 prefix not found\n");
         return;
      }
      break;
   default:
   	printf("Invalid phone number (wrong number of digits: %d)\n",
      	strlen(out));
      return;
   }

   /*
    * we now have the phone number parsed
    * display the number and comment on what type of number
    */
   if (areacode == DefaultAreaCode)
   {
   	printf("Phone number is local call %3.3d-%4.4d\n",
      	exchange, number);
   }
   else if (areacode == 800 || areacode == 801)
   {
   	printf("Phone number is a toll free call: 1-%3.3d-%3.3d-%4.4d\n",
      	areacode, exchange, number);
   }
   else if (areacode == 900)
   {
   	printf("You will be charged extra to call: 1-%3.3d-%3.3d-%4.4d\n",
      	areacode, exchange, number);
   }
   else
   {
   	printf("Long distance call: 1-%3.3d-%3.3d-%4.4d\n",
      	areacode, exchange, number);
   }
}

void
parse_help(void)
{
	printf("This option allows you to parse a line of text into words.\n");
   printf("It also allows you to parse dates and phone numbers\n");
   printf("Dates may be entered as:\n");
   printf("\t9/5/97\n");
   printf("Phone numbers can be entered as:\n");
   printf("\t(302)555-1234\n");
   printf("\nThe date parser is very fussy about the format\n");
   printf("but the phone number parser is very forgiving\n");
   printf("\nTry entering date as 12-21-97 and see what happens\n");
   printf("(can you figure out how to change the program so it will work?\n");
   printf("\nTry entering phone number using different formats\n");
}

