/*
 *				**** SEARCH ****
 *
 * This program shows you various ways
 * of dealing with arrays.
 *
 * For simplicity, the samples use an integer array,
 * but normally you will be working with arrays
 * of structures.  See the datademo program for
 * more details on how to do that.
 *
 */

#include <stdio.h>
#include <stdlib.h>

/* the maximum size of the table */
#define MAX_ENTS 100
#define MAX_LIST 10

/* the actual array we are working with */
int Array[MAX_ENTS], num_ents = 0, max_ents = MAX_ENTS;
char inbuf[BUFSIZ]; /* general input buffer from user */

/* function prototypes */
int enter_array(int array[], int n, int m);
int generate_array(int array[], int n, int m);
int find_first(int array[], int n, int key);
int find_last(int array[], int n, int key);
int count_match(int array[], int n, int key);
int list_match(int array[], int n, int key, int list[], int m);
void display_array(int array[], int beg, int end);

int
main(int argc, char **argv)
{
	char buf[BUFSIZ];
	int i, beg, end, opt, num, key, List[MAX_LIST];

	do
	{
		printf("\nOptions are:\n");
		printf(" 1. Clear array\n");
		printf(" 2. Enter number into array\n");
		printf(" 3. Generate random numbers for array\n");
		printf("\n");
		printf("11. Search for first match\n");
		printf("12. Search for last match\n");
		printf("13. Count number of matches\n");
		printf("14. Get list of matches\n");
		printf("\n");
		printf("91. Display array\n");
		printf("99. Quit\n");
		printf("Enter option: ");
		gets(buf);
		opt = atoi(buf);

		/* process option */
		switch(opt)
		{
		case  1:	/* clear array */
			printf("Clear the array (y or n): ");
			gets(buf);
			if (buf[0] == 'y')
			{ /* clear the array */
				num_ents = 0; /* that's all folks! */
			}
			break;

		case  2:	/* manually enter entries into array */
			num_ents = enter_array(Array, num_ents, max_ents);
			break;
		case  3:	/* generate random numbers into array */
			num_ents = generate_array(Array, num_ents, max_ents);
			break;

		case 11:	/* find first match in array */
			printf("Enter key: ");
			gets(buf);
			key = atoi(buf);
			num = find_first(Array, num_ents, key);
			if (num < 0)
			{ /* nothing found */
				printf("No match found\n");
			}
			else
			{
				printf("Match found at location %d\n", num);
			}
			break;
		case 12:	/* find last match in array */
			printf("Enter key: ");
			gets(buf);
			key = atoi(buf);
			num = find_last(Array,num_ents, key);
			if (num < 0)
			{
				printf("No match found\n");
			}
			else
			{
				printf("Match found at loc %d\n", num);
			}
			break;
		case 13:	/* count number of matches in array */
			printf("Enter key: ");
			gets(buf);
			key = atoi(buf);
			num = count_match(Array, num_ents, key);
			printf("%d match(es) found\n", num);
			break;
		case 14:	/* list all matches in array */
			printf("Enter key: ");
			gets(buf);
			key = atoi(buf);
			num = list_match(Array, num_ents, key, List, MAX_LIST);
			printf("%d match(es) found\n", num);
			for (i=0; i < num; i++)
			{
				printf("Match %d at location %d\n", i, List[i]);
			}
			break;

		case 91:	/* list entries in array */
			printf("%d entries in the array\n", num_ents);
         beg = 0;	/* default starting location */
			printf("Enter beginning location [%d]: ", beg);
			gets(buf);
			if (buf[0] != '\0') beg = atoi(buf); /* use non-default */
         end = num_ents - 1;	/* default end position */
			printf("Enter ending location [%d]: ", end);
			gets(buf);
			if (buf[0] != '\0') end = atoi(buf);	/* use val */
			display_array(Array, beg, end);
			break;
		case 99: /* quit */
			printf("Are you sure you want to quit? :");
			gets(buf);
			if (buf[0] != 'y') opt = 0;
			break;
		default:	/* unknown menu option */
			printf("Invalid option (%s) Try again\n", buf);
		}
      printf("Press ENTER to continue: ");
      gets(buf);
	} while (opt != 99);

	printf("Bye!\n");
	return 0;
}

/*
 *			**** FIND FIRST ****
 *
 * This routine searches the array for the first match
 * It returns the location of the first match
 * If not found, it returns -1
 *
 */
int
find_first(
	int array[],	/* table to search */
	int nents,		/* how many entrie in use */
	int key)			/* key to search for */
{ /* find first match in array, size nents */
	int i;

   /*
    * Loop thru table until we find a match
    */
	for (i=0; i < nents; i++)
	{ /* as soon as we find a match, return */
		if (array[i] == key) return i;
	}
	/* not found - return invalid value */
	return -1;
}

/*
 *			**** FIND LAST ****
 *
 * This routine will search the array looking for the
 * last entry matching the specified key
 *
 * Since we know how many entries are in the array,
 * we could have written this similar to find_first,
 * except that we search backwards:
 *
 *		for (i=num_ents-1; i >= 0; i--)
 *		{
 *			if (array[i] == key) return i;
 *		}
 *
 * However, we often don't know how many entries there are
 * This technique can be used in those cases
 */
int
find_last(
	int array[],	/* table to search */
	int n,			/* how many entries */
	int key)			/* key to find */
{
	int i, ret=-1;	/* location to return */

	for (i=0; i < n; i++)
	{
		if (array[i] == key) ret = i;
	}

	return ret; /* return last loc (or -1) */
}

/*
 *			**** COUNT MATCH ****
 *
 * This routine returns the number of entries that
 * were found in array that match the key
 */
 int
count_match(int array[], int n, int key)
{ /* see above for argument description */
	int i, ret=0;

	for (i=0; i < n; i++)
	{
		if (array[i] == key) ret++;
	}

	return ret;
}

/*
 *				**** LIST MATCH ****
 *
 * This routine returns a list of entry locations
 * that match the given key
 *
 * This routine returns the number of matches
 */
int
list_match(
	int array[],		/* table to search */
	int n,			/* how many entries */
	int key,			/* key to search for */
	int list[],		/* store results in list */
	int m)			/* max size of list */
{ /* return how many matches found */
	int i, ret=0;

	for (i=0; i < n; i++)
	{
		if (array[i] == key)
		{ /* match found - add to list */
			if (ret < m)
			{ /* room to add */
				list[ret] = i;	/* location of match */
				ret++;			/* how many matches */
			}
			else
			{ /* list is full */
				printf("List is full.  Not adding loc %d\n", i);
			}
		}
	}

	return ret;
}

/*
 *			**** DISPLAY ARRAY ****
 *
 * This routine displays a range of entries in the array
 */
void
display_array(int array[], int beg, int end)
{
	int i;
	char buf[BUFSIZ];

	for (i=beg; i < end; i++)
	{
		printf("Loc %d: %d\n", i, array[i]);
      if ((i%20) == 19)
      {
      	printf("Press ENTER to continue: ");
         gets(buf);
      }
	}
}

/*
 *				**** ENTER ARRAY ****
 *
 * This routine allows the user to manually add entries
 * to the array
 */
int
enter_array(
	int array[],		/* array to add to */
   int num,				/* number entries currently in array */
   int max)				/* max. entries in table */
{ /* return new number of entries */
	char buf[BUFSIZ];
   int n;

   n = num;				/* how many we have so far */
   while (n < max)
   {
   	printf("Enter number, or ENTER to quit: ");
      gets(buf);
      if (buf[0] == '\0') break;
      array[n] = atoi(buf);
      n++;
   }
	return n;
}

int
generate_array(
	int array[],		/* array to be updates */
   int num,				/* current number of entries */
   int max)				/* max. number of entries */
{ /* return new number of entries */
	char buf[BUFSIZ];
   int n, a, m;

   n = num;				/* how many entries so far */

   printf("There are currently %d out of %d entries in use\n",
   	n, max);
   printf("How many numbers do you want in the table: ");
   gets(buf);
   a = atoi(buf);

   printf("What is the largest number you want: ");
   gets(buf);
   m = atoi(buf);			/* numbers will be less than this */
   if (m <= 0) return;	/* must be positive integer */

   while (n < a)
   { /* fill up the table */
   	/*
       * we use the mod operator (%) to restrict values
       * from zero to 'm'
       */
   	array[n] = rand() % m;	/* add random number */
      n++;							/* one more entry */
   }

	return n;
}

